;*******************************************************************************
;*
;*      Titel:  Unterprogramm zur 12-Tasten-Matrix-Tastatur (SR_3x4_KEYPAD.asm)
;*
;*      Datum:  23/12/07        Version:        0.1     
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32        Quarzfrequenz:  16MHz 
;*      Eingaenge:      3 Spalten (columns) sind Eingaenge am Port (2^4-2^6)
;*                      interne Pull-Ups muessen eingeschaltet werden
;*      Ausgaenge:      4 Zeilen (rows) sind Ausgaenge am Port (2^0-2^3)
;*
;*      Informationen zur Funktionsweise:
;*      
;*      Im Hauptprogramm muss das Port (KeyPort, KeyPin) initialisiert werden.
;*      Das fuer den Tastenwert zustaendige Arbeitsregister (Tastenzaehler) muss
;*      dem Namen KeyCode zugewiesen werden (globale Variable).
;*      Die Zeitschleifenbibliothek muss eingebunden sein.
;*      
;*      Spalten sind Eingaenge und liegen ueber Pull-Up auf High. Zeilen sind
;*      Ausgaenge und werden zur Aktivierung auf Low gelegt. Gedrueckte Taste
;*      zieht Spalte auf Low!
;*      In "KeyCode" wird die gedruekte Taste uebermittelt. 10 entspricht dem
;*      Stern, 11 der Null und 12 der Raute. Bei 0 wurde keine Taste gedrueckt!
;*                      
;*******************************************************************************

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU	KeyPort = PORTA
.EQU	KeyPin = PINA
.EQU	KeyDDR = DDRA
.EQU  KP_C1  = 0                ;Tastatur 1 Spalte (147*) Eingang
.EQU  KP_C2  = 1                ;Tastatur 2 Spalte (2580) Eingang
.EQU  KP_C3  = 2                ;Tastatur 3 Spalte (369#) Eingang
.EQU  KP_R1  = 3                ;Tastatur 1. Zeile (123) Ausgang 
.EQU  KP_R2  = 4                ;Tastatur 2. Zeile (456) Ausgang 
.EQU  KP_R3  = 5                ;Tastatur 3. Zeile (789) Ausgang 
.EQU  KP_R4  = 6                ;Tastatur 4. Zeile (*0#) Ausgang 

;-------------------------------------------------------------------------------
;       Unterprogramm Keypad initialisieren
;-------------------------------------------------------------------------------
KEYINI: push	r16

	in	r16,KeyDDR
	ori	r16,0x78	;4 Ausgaenge
	andi	r16,0xF8	;3 Eingaenge
	out	KeyDDR,r16
	in	r16,KeyPORT
	ori	r16,0x07	;3xPull-Up	
	out	KeyPORT,r16
	
	pop	r16
	ret
	

;-------------------------------------------------------------------------------
;       Unterprogramm Tastatur
;-------------------------------------------------------------------------------
KEYP:   push    r16             ;r16 retten
        clr     r16	        ;r16 = DebounceCounter dient dazu zu erkennen ob
                                ;schon entprellt wurde (kann auch > 1 sein)
KEYP1:  ldi     KeyCode,12      ;Initialisiere Tastenzaehler KeyCode mit 12  
        ;Ausgangszustand initialisieren (4. Zeile aktivieren)
        sbi     KeyPort,KP_R1   ;1. Zeile auf High schalten (inaktiv)
        sbi     KeyPort,KP_R2   ;2. Zeile auf High schalten (inaktiv)
        sbi     KeyPort,KP_R3   ;3. Zeile auf High schalten (inaktiv)
        cbi     KeyPort,KP_R4   ;4. Zeile aktivieren (Low)
        ;drei Spalten abfragen
KEYP2:  sbis    KeyPin,KP_C3    ;3. Spalte abfragen; Weiter bei High
        rjmp    KEYP5           ;(not pressed) sonst KEYP5
        dec     KeyCode         ;erniedrige Tastenzaehler KeyCode
        sbis    KeyPin,KP_C2    ;2. Spalte abfragen; Weiter bei High
        rjmp    KEYP5           ;(not pressed) sonst KEYP5
        dec     KeyCode         ;erniedrige Tastenzaehler KeyCode
        sbis    KeyPin,KP_C1    ;3. Spalte abfragen; Weiter bei High
        rjmp    KEYP5           ;(not pressed) sonst KEYP5
        dec     KeyCode         ;erniedrige Tastenzaehler KeyCode
        ;3. Zeile aktivieren
        sbis    KeyPin,KP_R3    ;Falls 3. Zeile nicht auf Low dann aktivieren
        rjmp    KEYP3           ;Falls schon auf Low dann weiter mit KEYP3
        cbi     KeyPort,KP_R3   ;3. Zeile auf Low
        rjmp    KEYP2           ;Spalten testen
        ;2. Zeile aktivieren
KEYP3:  sbis    KeyPin,KP_R2    ;Falls 2. Zeile nicht auf Low dann aktivieren
        rjmp    KEYP4           ;Falls schon auf Low dann weiter mit KEYP4
        cbi     KeyPort,KP_R2   ;2. Zeile auf Low
        rjmp    KEYP2           ;Spalten testen
        ;1. Zeile aktivieren
KEYP4:  sbis    KeyPin,KP_R1    ;Falls 1. Zeile nicht auf Low dann aktivieren
        rjmp    KEYP6           ;Falls schon auf Low dann fertig mit UP
        cbi     KeyPort,KP_R1   ;1. Zeile auf Low
        rjmp    KEYP2           ;Spalten testen
        ;Pruefen ob schon entprellt wurde
KEYP5:  tst     r16             ;Falls gleich Null, bereits entprellt UP beenden
        breq    KEYP6           ;
        rcall   W1ms            ;Falls nicht gleich, 1ms warten (entprellen) 
        dec     r16             ;Debouncecounter dekrementieren            
        rjmp    KEYP1           ;Tasten noch einmal abfragen
KEYP6:  pop     r16             ;r16 wiederherstellen
        ret                     ;fertig mit UP       
