;*******************************************************************************
;*
;*      Titel:  Unterprogramm zur Nutzung des RTC DS1307 ueber I2C
;*              (SR_RTC_DS1307.asm)
;*
;*      Datum:  01/01/11        Version:        0.3
;*      Autor:  WEIGU
;*
;*      Informationen zur Beschaltung:
;*      Prozessor:      ATmega32A        Quarzfrequenz:  16MHz
;*      Eingaenge:
;*      Ausgaenge:
;*
;*      Informationen zur Funktionsweise:
;*
;*
;*******************************************************************************
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
;       Reservierungen und Zuweisungen
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
.EQU    ERRLED   = 0            ;1 falls Fehler LED erwuenscht sons 0
.EQU    ERRDDR   = DDRD         ;LED zur Anzeige eines Uebetragungsfehler
.EQU    ERRPORT  = PORTD
.EQU    ERRPNr   = 7

;-------------------------------------------------------------------------------
;	Daten senden (8 Datenbyte)
;-------------------------------------------------------------------------------
RTCSND: rcall	I2CSTA		;Startbit senden + Kontrolle
        cpi     r25,0x08
        brne    RTCSNE
        ldi	r24,0xD0	;Adresse des RTC DS1307 (Maxim 1101000) senden
	rcall	I2CSND		;(Bit 2^0 = 0 schreiben) + Kontrolle
        cpi     r25,0x18
        brne    RTCSNE
	ldi	r24,0x00	;Adresszeiger = 0 senden + Kontrolle
	rcall	I2CSND
        cpi     r25,0x28        
        brne    RTCSNE
        ;8 Datenbyte senden + Kontrolle
        lds     r24,RTCTAB+0    ;Sekunden
        rcall   I2CSND
        cpi     r25,0x28        
        brne    RTCSNE
        lds     r24,RTCTAB+1    ;Minuten
        rcall   I2CSND
        cpi     r25,0x28
        brne    RTCSNE
        lds     r24,RTCTAB+2    ;Stunden
        rcall   I2CSND
        cpi     r25,0x28
        brne    RTCSNE
        lds     r24,RTCTAB+3    ;Wochentag
        rcall   I2CSND
        cpi     r25,0x28
        brne    RTCSNE
        lds     r24,RTCTAB+4    ;Tag
        rcall   I2CSND
        cpi     r25,0x28
        brne    RTCSNE
        lds     r24,RTCTAB+5    ;Monat
        rcall   I2CSND
        cpi     r25,0x28
        brne    RTCSNE
        lds     r24,RTCTAB+6    ;Jahr
        rcall   I2CSND
        cpi     r25,0x28
        brne    RTCSNE
        lds     r24,RTCTAB+7    ;Kontrollregister
        rcall   I2CSND
        cpi     r25,0x28
        brne    RTCSNE
        rjmp    RTCSNR
RTCSNE: .IF (ERRLED == 1)
        sbi     ERRDDR,ERRPNr   ;Ausgang fuer Fehler LED initialisieren
        sbi     ERRPORT,ERRPNr  ;Fehler LED ein
        .ENDIF
RTCSNR: rcall   I2CSTO          ;Stoppbit senden
        ret

;-------------------------------------------------------------------------------
;	Daten empfangen
;-------------------------------------------------------------------------------
RTCRCV: rcall	I2CSTA		;Startbit senden + Kontrolle
        cpi     r25,0x08        
        breq    RTCRC1
        cpi     r25,0x10        ;(repeated start falls zu schnell fuer Stoppbit)
        brne    RTCRCE
RTCRC1:	ldi	r24,0xD0	;Adresse des RTC DS1307 (Maxim 1101000) senden
	rcall	I2CSND		;(Bit 2^0 = 0 schreiben) + Kontrolle
        cpi     r25,0x18        
        brne    RTCRCE
	ldi	r24,0x00	;Adresszeiger = 0 senden + Kontrolle
	rcall	I2CSND
        cpi     r25,0x28        
        brne    RTCRCE
       	rcall	I2CSTA		;Repeated Startbit senden + Kontrolle
        cpi     r25,0x10
        brne    RTCRCE
	ldi	r24,0xD1	;Adresse des RTC DS1307 (Maxim 1101000) senden
	rcall	I2CSND		;(Bit 2^0 = 1 lesen) + Kontrolle
        cpi     r25,0x40
        brne    RTCRCE
        ;8 Datenbyte lesen + Kontrolle
	rcall	I2CRAK          ;Sekunden
        sts     RTCTAB+0,r24
        cpi     r25,0x50
        brne    RTCRCE
        rcall   I2CRAK          ;Minuten
        sts     RTCTAB+1,r24
        cpi     r25,0x50
        brne    RTCRCE
        rcall   I2CRAK          ;Stunden
        sts     RTCTAB+2,r24
        cpi     r25,0x50
        brne    RTCRCE
        rcall   I2CRAK          ;Wochentag
        sts     RTCTAB+3,r24
        cpi     r25,0x50
        brne    RTCRCE
        rcall   I2CRAK          ;Tag
        sts     RTCTAB+4,r24
        cpi     r25,0x50
        brne    RTCRCE
        rcall   I2CRAK          ;Monat
        sts     RTCTAB+5,r24
        cpi     r25,0x50
        brne    RTCRCE
        rcall   I2CRAK          ;Jahr
        sts     RTCTAB+6,r24
        cpi     r25,0x50
        brne    RTCRCE
        rcall   I2CRNA          ;Kontrollregister Achtung! mit NAK abschliessen
        sts     RTCTAB+7,r24
        cpi     r25,0x58
        brne    RTCRCE
        rjmp    RTCRCR
RTCRCE: .IF (ERRLED == 1)
        sbi     ERRDDR,ERRPNr   ;Ausgang fuer Fehler LED initialisieren
        sbi     ERRPORT,ERRPNr  ;Fehler LED ein
        .ENDIF
RTCRCR: rcall   I2CSTO          ;Stoppbit senden
        ret
        
;-------------------------------------------------------------------------------
;	Datum und Zeit zur seriellen Schnittstelle senden (ASCII; Unterprogramm
;	SNDCHR muss existieren) und Abspeichern in SRAM Tabelle
;	(RTCASCT 19 Byte: dd.mm.yy hh.MM.ss+Space+NullByte)
;-------------------------------------------------------------------------------
SNDTD:  lds     r24,RTCTAB+4    ;Tag
	rcall	BCDTOA
	sts	RTCASC+0,r25
	sts	RTCASC+1,r24
	ldi	r24,'.'	
	sts	RTCASC+2,r24
	rcall	SNDCHR
	lds     r24,RTCTAB+5    ;Monat
	rcall	BCDTOA
	sts	RTCASC+3,r25
	sts	RTCASC+4,r24
	ldi	r24,'.'
	sts	RTCASC+5,r24
	rcall	SNDCHR
	lds     r24,RTCTAB+6    ;Jahr
	rcall	BCDTOA
	sts	RTCASC+6,r25
	sts	RTCASC+7,r24
	ldi	r24,' '
	sts	RTCASC+8,r24
	rcall	SNDCHR
  	lds     r24,RTCTAB+2    ;Stunden
	rcall	BCDTOA
	sts	RTCASC+9,r25
	sts	RTCASC+10,r24
	ldi	r24,':'
	sts	RTCASC+11,r24
	rcall	SNDCHR
	lds     r24,RTCTAB+1    ;Minuten
	rcall	BCDTOA
	sts	RTCASC+12,r25
	sts	RTCASC+13,r24
	ldi	r24,':'
	sts	RTCASC+14,r24
	rcall	SNDCHR
	lds     r24,RTCTAB+0	;Sekunden
	rcall	BCDTOA
	sts	RTCASC+15,r25
	sts	RTCASC+16,r24
	ldi	r24,' '
	sts	RTCASC+17,r24
	rcall	SNDCHR	
	clr	r24		;Nullbyte
	sts	RTCASC+18,r24
	ret
	
BCDTOA:	push	r16
	push	r17
	ldi	r16,0x30	
	mov	r17,r24
	andi	r24,0xF0
	swap	r24	
	add	r24,r16
	rcall	SNDCHR	
	mov	r25,r24
	mov	r24,r17
	andi	r24,0x0F
	add	r24,r16
	rcall	SNDCHR	
	pop	r17
	pop	r16
	ret
	